@@ -2,20 +2,22 @@ CACHE = (("file_name_in",),{'type':str,'help':"read map data from cache file CAC
 LIM = (("file_name_in",),{'type':str,'help':"read map data from LiMiC file FILE",'metavar':'LIM'})
 OSM = (("file_name_in",),{'type':str,'help':"read map data from OpenStreetMap file OSM",'metavar':'OSM'})
 OUT = (("file_name_out",),{'type':str,'help':"save extracted graph to file GRAPH",'metavar':'GRAPH'})
-EPS = (("-e","--epsilon"),{'type':float,'dest':'eps','default':0.01,'help':"set minimum percentage of power line segment to consider non-logical intersections to EPS",'metavar':'EPS'})
-AROUND = (("-r","--around"),{'type':int,'dest':'around','default':1000,'help':"set maximum free flying distance in m around power towers to AROUND",'metavar':'AROUND'})
-SAFEDIST = (("-s","--safe-dist"),{'type':int,'dest':'safe_dist','default':100,'help':"set minimum safe distance from transformer stations and other perilous structures to SAFE",'metavar':'SAFE'})
-PENALIZE = (("-p","--penalize"),{'type':int,'default':20,'dest':'penalize','help':"penalize air time by factor PENALTY",'metavar':'PENALTY'})
+EPS = (("-e","--epsilon"),{'type':float,'dest':'eps','default':0.01,'help':"set minimum percentage of  linear infrastructure segment to consider non-logical intersections to EPS",'metavar':'EPS'})
+AROUND = (("-r","--around"),{'type':int,'dest':'around','default':1000,'help':"set maximum free flying distance in m around linear infrastructure nodes to AROUND",'metavar':'AROUND'})
+SAFEDIST = (("-s","--safe-dist"),{'type':int,'dest':'safe_dist','default':100,'help':"set minimum safe distance from blacklisted areas and structures to SAFE",'metavar':'SAFE'})
+PENALIZE = (("-p","--penalize"),{'type':int,'default':20,'dest':'penalize','help':"penalize free flying disrances by factor PENALTY",'metavar':'PENALTY'})
+WHITE = (("--white",),{'type':str,'dest':'white','default':'{"power":"line"}','help':"linear infrastructure to include (default: {'power':'line'})",'metavar':'WHITELIST'})
+BLACK = (("--black",),{'type':str,'dest':'black','default':'{"power":"substation"}','help':"areas and structures to avoid (default: {'power':'substation'})",'metavar':'BLACKLIST'})
 #CONSERVEMEM = (("-c","--conserve-memory"),{'action':'store_true','dest':'conserve_mem','default':False,'help':"lower memory usage but higher runtime"})
 
 CONFIG = [
-    ("nx",{'args':[
+    ("cache",{'args':[
         (("-u","--overpass-url"),{'type':str,'dest':'overpass_url','help':"define url for Overpass API to be URL",'metavar':'URL'}),
         (("-a","--area"),{'type':str,'dest':'area','help':"set area to extract to AREA",'metavar':'AREA'}),
         AROUND,EPS,SAFEDIST,PENALIZE,CACHE,OUT
     ]}),
-    ("osm",{'args':[AROUND,EPS,SAFEDIST,PENALIZE,OSM,OUT]}),
-    ("osm_pre",{'args':[OSM,OUT]}),
+    ("osm",{'args':[BLACK,WHITE,AROUND,EPS,SAFEDIST,PENALIZE,OSM,OUT]}),
+    ("osm_pre",{'args':[BLACK,WHITE,OSM,OUT]}),
     ("osm_post",{'args':[AROUND,EPS,SAFEDIST,PENALIZE,LIM,OUT]})
     #("esy",{'args':[OSM,OUT]})
 ]
@@ -67,11 +69,12 @@ def build_edges(g,find_all_neighbours,around,eps,safe_dist,penalize):
                     neighbours2intersection[neighbour.tower.neighbours].append(neighbour.tower)
 
 
-def extract_nx(file_name_in,file_name_out,overpass_url,area=None,around=1000,eps=0.01,safe_dist=100,penalize=20):
+def extract_cache(file_name_in,file_name_out,overpass_url,area=None,around=1000,eps=0.01,safe_dist=100,penalize=20):
     from limic.overpass import distance, find_all_neighbours, is_safe, set_server, pylon, region, get_towers_by_area
-    from limic.util import start, end, file_size, status, load_pickled, save_pickled, verbosity
+    from limic.util import start, end, file_size, status, load_pickled, save_pickled, verbosity, replace, check_overwrite
     from networkx import Graph
-    from os import replace
+    if not check_overwrite(file_name_in,file_name_out):
+        return
     start("Loading",file_name_in)
     region.backend._cache = load_pickled(file_name_in)
     len_cache = len(region.backend._cache)
@@ -210,20 +213,28 @@ def find_all_neighbours(tower,around,eps,safe_dist,minusid,latlon2id,penalize,m_
             #if verbosity >= 4: print("unsafe and skipped:",neighbour.tower)
     return safe_neighbours
 
-def extract_osm(file_name_in,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20):
-    lim = osm_pre(file_name_in)
+def extract_osm(file_name_in,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20,white="{'power':'line'}",black="{'power':'substation'}"):
+    from limic.util import check_overwrite
+    if not check_overwrite(file_name_in,file_name_out):
+        return
+    lim = osm_pre(file_name_in,white,black)
     osm_post(lim,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20)
 
-def extract_osm_pre(file_name_in,file_name_out): #,conserve_mem):
-    from limic.util import start, end, save_pickled, file_size
-    lim = osm_pre(file_name_in)
+def extract_osm_pre(file_name_in,file_name_out,white="{'power':'line'}",black="{'power':'substation'}"): #,conserve_mem):
+    from limic.util import start, end, save_pickled, file_size, check_overwrite
+    if not check_overwrite(file_name_in,file_name_out):
+        return
+    white,black = eval(white).items(), eval(black).items()
+    lim = osm_pre(file_name_in,white,black)
     start("Saving data to",file_name_out)
     save_pickled(file_name_out,lim)
     end('')
     file_size(file_name_out)
 
 def extract_osm_post(file_name_in,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20):
-    from limic.util import start, end, file_size, load_pickled
+    from limic.util import start, end, file_size, load_pickled, check_overwrite
+    if not check_overwrite(file_name_in,file_name_out):
+        return
     start("Loading filtered OSM data from",file_name_in)
     lim = load_pickled(file_name_in)
     end('')
@@ -283,7 +294,7 @@ def osm_post(lim,file_name_out,around=1000,eps=0.01,safe_dist=100,penalize=20):
     status(len(g.edges()))
     common_end(g,file_name_out)
 
-def osm_pre(file_name_in):
+def osm_pre(file_name_in,white=[("power","line")],black=[("power","substation")]):
     from limic.util import start, end, status, file_size, save_pickled
     from xml.sax import parse
     from xml.sax.handler import ContentHandler
@@ -302,20 +313,20 @@ def osm_pre(file_name_in):
                 self.nodes = []
                 self.power = None
             elif name == 'tag':
-                if attrs['k'] == 'power':
-                    if attrs['v'] == 'line':
-                        self.power = "line"
-                    elif attrs['v'] == 'substation':
-                        self.power = "substation"
+                kv = (attrs['k'],attrs['v'])
+                if kv in white:
+                    self.power = "white"
+                elif kv in black:
+                    self.power = "black"
             elif name == 'nd':
                 self.nodes.append(int(attrs['ref']))
             elif name == 'node':
                 self.towers.append(pylon(int(attrs['id']),(float(attrs['lat']),float(attrs['lon']))))
         def endElement(self, name):
             if name == 'way':
-                if self.power == 'line':
+                if self.power == 'white':
                     self.lines.append(self.nodes)
-                elif self.power == 'substation':
+                elif self.power == 'black':
                     self.substations.append(self.nodes)
     f = bopen(file_name_in,"rb") if file_name_in[-4:] == ".bz2" else open(file_name_in,"rb")
     ways = ways_handler()
