@@ -11,12 +11,14 @@ CONFIG = [
 ]
 
 def render_nx(file_name_in,file_name_out,markers=False,lines=False,host="localhost",port=5000,prefix=""):
-    from limic.util import start, end, status, file_size, load_pickled
+    from limic.util import start, end, status, file_size, load_pickled, check_overwrite
     from folium import Map, Marker, Icon, PolyLine
     from folium.plugins import BeautifyIcon
     from binascii import hexlify
     from pathlib import Path
     from math import log2
+    if not check_overwrite(file_name_in,file_name_out):
+        return
     start("Loading NX graph",file_name_in)
     g = load_pickled(file_name_in)
     end()
